/***********************************************************
 *
 *   Program:  PaellaMF30      			Version: 1.00
 *
 *   Author:  S. Swift					Date: Nov 14, 2014
 *
 *   PaellaMF30 application configures the PicoM with STM32F303 as a CDC USB device, 10DOF and
 *   Wireless interface hub controller.  The application was developed using the free
 *   CooCox V1.7.8 IDE and STMicroelectronics STM32F303_USB-FS-Device_Lib V1.0.0.
 *
 *   This software can be used without restriction as the basis for other applications
 *   and products.  The STM32F303 is a low cost ($6.50) high performance ARM M4 processor
 *   with hardware FPU running at 72MHz with 256K of flash, 44K SRAM, A/D, D/A, SPI, USARTs,
 *   I2C, HDMI, etc in a 48 pin QFP package. The Wireless port accepts a nRF24L01 modem.
 *
 *   The application configures the USB peripheral as a CDC device, USART3 to 38,400,N,8,1,
 *   TIM3 as 1msec time delay clock, sets up a few other timers and implements a printf,scanf
 *   for USB CDC and printf3 for USART3.  USART3 is interrupt driven.  Functions to control the RF24L01
 *   is implemented along with functions to support the MPU-9150 9 DOF sensor and BMP180 barometer.
 *   Functions to support a low cost ST7735 color LCD display is also included.
 *
 *   The Wireless connection is managed as a Master to allow communication with other Nano's and
 *   Picos.  An enumeration function determines what devices are in the network and returns identification\
 *   data.
 *
 *   The PaellaMF30 has two I2C ports muxed out to five separate I2C ports through a 1:4 mux selector
 *   designed to hard connect with Femto Sensor Nodes such as the 6DOF FemtoB.
 *
 *   Unless required by applicable law or agreed to in writing, software distributed under the
 *   License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 *   either express or implied.  See the License for the specific language governing permissions
 *   and limitations under the License. Use at your own risk.
 *
 *   Copyright 2014,  SOC Robotics, Inc.
 *******************************************************************************/
/**
  ******************************************************************************
  * @file    main.c
  * @author  MCD Application Team
  * @version V4.0.0
  * @date    21-January-2013
  * @brief   Virtual Com Port Demo main file
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; COPYRIGHT 2013 STMicroelectronics</center></h2>
  *
  * Licensed under MCD-ST Liberty SW License Agreement V2, (the "License");
  * You may not use this file except in compliance with the License.
  * You may obtain a copy of the License at:
  *
  *        http://www.st.com/software_license_agreement_liberty_v2
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "hw_config.h"
//#include "usbd_cdc_core.h"
//#include "usbd_usr.h"
#include "usb_lib.h"
#include "usb_desc.h"
#include "usb_pwr.h"
#include "usbd_cdc_vcp.h"

#include "string.h"
#include "math.h"
#include "stdarg.h"
#include "stdio.h"
#include "stdlib.h"

#include "PicoM_Defines.h"
#include "PicoM_Externals.h"
#include "NRF24L01.h"
#include "FXOS8700.h"
#include "MMA8653.h"
#include "SI7013.h"
#include "st7735.h"
#include "colors.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Extern variables ----------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

#define USE_FULL_ASSERT

#define RXBUFSIZE 100

#define LED0ON  GPIO_ResetBits(GPIOB,GPIO_Pin_0);
#define LED0OFF  GPIO_SetBits(GPIOB,GPIO_Pin_0);

#define LED1HIGH   GPIOB->BSRR = GPIO_Pin_0
#define LED1LOW	   GPIOB->BRR = GPIO_Pin_0

#define PA6HIGH   GPIOA->BSRR = GPIO_Pin_6
#define PA6LOW	  GPIOA->BRR = GPIO_Pin_6
#define PA7HIGH   GPIOA->BSRR = GPIO_Pin_7
#define PA7LOW	  GPIOA->BRR = GPIO_Pin_7

#define SCK2LOW		GPIO_ResetBits(GPIOB,GPIO_Pin_13)
#define SCK2HIGH	GPIO_SetBits(GPIOB,GPIO_Pin_13)
#define MOSI2LOW	GPIO_ResetBits(GPIOB,GPIO_Pin_15)
#define MOSI2HIGH	GPIO_SetBits(GPIOB,GPIO_Pin_15)

// STM32F303 defines
#define PWMACTIVE  'a'
#define CMDACTIVE  'c'
#define INACTIVE 'i'

#define I2C100KHZ  0x20310A0D
//#define I2C400KHZ  0x00310A0D  // old
#define I2C400KHZ 0x00A00102
//#define I2C400KHZ  0x00E00D38
//#define I2C1000KHZ 0x00310A0D
//#define I2C1000KHZ 0x00A00D26
#define I2C1000KHZ 0x00000000
//#define I2C1000KHZ 0x00400d2c


/* Uncomment the following line to use Timeout_User_Callback I2C_TimeoutUserCallback().
   If This Callback is enabled, it should be implemented by user in main function .
   TS751_TimeoutUserCallback() function is called whenever a timeout condition
   occure during communication (waiting on an event that doesn't occur, bus
   errors, busy devices ...). */
#define USE_TIMEOUT_USER_CALLBACK

/* Maximum Timeout values for flags and events waiting loops. These timeouts are
   not based on accurate values, they just guarantee that the application will
   not remain stuck if the I2C communication is corrupted.
   You may modify these timeout values depending on CPU frequency and application
   conditions (interrupts routines ...). */
//#define I2C_FLAG_TIMEOUT         ((uint32_t)0x1000)
#define I2C_LONG_TIMEOUT         ((uint32_t)(10 * I2C_FLAG_TIMEOUT))

/**
  * @brief  Timeout user callback function. This function is called when a timeout
  *         condition occurs during communication with TS751. Only protoype
  *         of this function is declared in TS751 driver. Its implementation
  *         may be done into user application. This function may typically stop
  *         current operations and reset the I2C peripheral and TS751.
  *         To enable this function use uncomment the define USE_TIMEOUT_USER_CALLBACK
  *         at the top of this file.
  */
#ifdef USE_TIMEOUT_USER_CALLBACK
 uint8_t I2C_TIMEOUT_UserCallback(void);
#else
 #define I2C_TIMEOUT_UserCallback()  TS751_FAIL
#endif /* USE_TIMEOUT_USER_CALLBACK */

volatile char flip=0,grnflip=0,redflip=0,ledblink;
uint16_t TimerPeriod = 0;
uint16_t Channel1Pulse = 0, Channel2Pulse = 0, Channel3Pulse = 0, Channel4Pulse = 0;

// Timer, counter stuff
volatile unsigned int mseccountdown1,mseccountdown2,mseccountdown3,mseccountdown4,bmp180countdown;
volatile unsigned long mseccounter=0,rf_cnt;
volatile int usectimertally=0;
volatile bool usbconnected=false;

volatile char memsstate=ACTIVESTATE;
volatile unsigned char messageBuf[20],i2ctimedout;
volatile unsigned int i2c_Timeout;

volatile float pressure,temperature;

// MPU-9150 9 DOF sensor parameters
volatile char mpu9150state=0;
volatile float magposition[6],mpu9150_tempoffset=MPU9150TEMPOFFSET;
extern volatile struct imudata {
	short int ai[3];
	short int gi[3];
	short int mi[3];
	short int temp;
	float magcor[3];
	float af[3];
	float gf[3];
	float mf[3];
	short int tempi;
	short int rolli;
	short int pitchi;
	short int yawi;
	short int pressi;
	float tempf;
	float roll;
	float pitch;
	float yaw;
	float press;
	char mget;
} imu;

// FXOS8700 6DOF 3-axis accelerometer, 3-axis magnetometer variables
volatile char fxos8700state=0;
volatile int fingerindex,digitindex;

FXOS_F dip[5];
FXOS_F pip[5];
FXOS_F mcp[5];

volatile FXOSRawData fxosRawData;
volatile char pca9545channel[4] = {PCA9545_CHANNEL_0,PCA9545_CHANNEL_1,PCA9545_CHANNEL_2,PCA9545_CHANNEL_3};
extern FXOS_F FXOS8700CQ_Accel_Mag_Float(I2C_TypeDef* I2Cx,unsigned char address);

// BMP180 Barometer Parameters
volatile unsigned char BMPstate=0,bmp180mode,bmp180tempstate,bmp180pressurestate,bmp180state;
volatile unsigned short int bmpac4,bmpac5,bmpac6,ibmptemperature,ibmppressure;
volatile short int  bmpac1,bmpac2,bmpac3,bmpb1,bmpb2,bmpmb,bmpmc,bmpmd;
volatile int  bmpskipcount;
volatile long b5;
volatile float bmppressure,bmptemperature,pressuremin,pressuremax,bmp180_tempoffset=BMP180TEMPOFFSET;

// MMA8653 3DOF accelerometer variables
volatile char mma8653state=0;

// SI7013 Humidity sensor variables
volatile char si7013state,si7013sample=SI7013START,si7013sampletarget=SI7013SAMPLEHUMIDITY;
volatile struct sid {
	unsigned int rawtemperature,rawhumidity;
	float temperature,humidity;
	} si7013;

// L3GD20 3DOF gyro sensor variables
volatile char l3gd20state=0;

// ADPS-9007 light sensor variables
volatile char adps9007state=0;

// Display configuration parameters - this is set at init time
volatile char updatedisplay=INACTIVE;
volatile int updaterate=0;

//#define Timed(x) Timeout = 0xFFFF; while (x) { if (Timeout-- == 0) goto errReturn;}


char rxbuf[RXBUFSIZE];
int rxheadptr,rxtailptr;

extern int printfu(const char *fmt, ...);
TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;
TIM_OCInitTypeDef  TIM_OCInitStructure;
TIM_ICInitTypeDef  TIM_ICInitStructure;

FILE *fpout;

#define     USB_RX_BUFFERSIZE 512

int usbrxheadptr=0, usbrxtailptr=0;
unsigned char usbrxbuffer[USB_RX_BUFFERSIZE];

#define     USART2_TX_BUFSIZE 80
#define     USART2_RX_BUFSIZE 80

int tx2headptr,tx2tailptr,rx2headptr,rx2tailptr;
unsigned char rx2buffer[USART2_RX_BUFSIZE],tx2buffer[USART2_TX_BUFSIZE];

uint16_t IC2Value = 0;
uint16_t DutyCycle = 0;
uint32_t Frequency = 0;

/***********************************************
 *
 *  Initialization Functions
 *
 ***********************************************/

// PA9 is connected to VBUS and is used to determine if
// the USB cable is connected.  If not connected then the
// USB subsystem is not started. PA9 is 5V tolerance.
void VBUS_Monitor_Init(void)
{
	GPIO_InitTypeDef  GPIO_InitStructure;

	// Enable the GPIO_LED Clock
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);

	// Configure PA9 and PA7 as outputs for testing
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_DOWN;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
}

//  Configures PA6 and P7 as outputs for testing purposes
void PortA_Init(void)
{
	GPIO_InitTypeDef  GPIO_InitStructure;

	// Enable the GPIO_LED Clock
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);

	// Configure the PA6 and PA7 as outputs for testing
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_7 | GPIO_Pin_6;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	//GPIO_InitStructure.GPIO_Speed = GPIO_Speed_Level_1 ;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
}

/**
  * @brief  Configures LED GPIO.
  */
void LED_Init(void)
{
	GPIO_InitTypeDef  GPIO_InitStructure;

	/* Enable the GPIO_LED Clock */
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);

	/* Configure the GPIO_LED pin */
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOB, &GPIO_InitStructure);
}
/**
  * @brief  Configures COM port.  Supports buffered interrupt IO
  */
void USART2_Init(unsigned long baudrate)
{
    GPIO_InitTypeDef GPIO_InitStructure;
    USART_InitTypeDef USART_InitStructure;
	NVIC_InitTypeDef NVIC_InitStructure;

    // Initialize interrupt tx rx buffers
    tx2headptr = tx2tailptr = rx2headptr = rx2tailptr = 0;

    /* USARTx configured as follow:
    - BaudRate = 115200 baud
    - Word Length = 8 Bits
    - One Stop Bit
    - No parity
    - Hardware flow control disabled (RTS and CTS signals)
    - Receive and transmit enabled
    */
    USART_InitStructure.USART_BaudRate = baudrate;
    //USART_InitStructure.USART_BaudRate = 115200;
    USART_InitStructure.USART_WordLength = USART_WordLength_8b;
    USART_InitStructure.USART_StopBits = USART_StopBits_1;
    USART_InitStructure.USART_Parity = USART_Parity_No;
    USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
    USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;

    /* Enable GPIO clock */
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);

    /* Enable USART clock */
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART2, ENABLE);

    /* Connect PXx to USARTx_Tx */
    GPIO_PinAFConfig(GPIOB, GPIO_PinSource3, GPIO_AF_4);

    /* Connect PXx to USARTx_Rx */
    GPIO_PinAFConfig(GPIOB, GPIO_PinSource4, GPIO_AF_4);

    /* Configure USART Tx, Rx as alternate function push-pull */
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_3 | GPIO_Pin_4;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
    GPIO_Init(GPIOB, &GPIO_InitStructure);

    /* USART configuration */
    USART_Init(USART2, &USART_InitStructure);

    USART_ITConfig(USART2, USART_IT_RXNE, ENABLE);
    USART_ITConfig(USART2, USART_IT_TXE, ENABLE);

	/* Enable the USART3 global Interrupt */
	NVIC_InitStructure.NVIC_IRQChannel = USART2_IRQn;
//	NVIC_InitStructure.NVIC_IRQChannelPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);

    /* Enable USART */
    USART_Cmd(USART2, ENABLE);
}

/**
  * @brief  Configure the TIM1 Pins.
  */
void TIM1_Config(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;

	/* GPIOA Clocks enable */
	RCC_AHBPeriphClockCmd( RCC_AHBPeriph_GPIOA, ENABLE);

	/* GPIOA Configuration: Channel 1, 2 as alternate function push-pull */
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8 | GPIO_Pin_11;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP ;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	GPIO_PinAFConfig(GPIOA, GPIO_PinSource8, GPIO_AF_2);
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource11, GPIO_AF_2);
}

// PWM output example - TIM2 can generate four different pwm outputs
void TIM1_ConfigPWM(void)
{
	/* Compute the value to be set in ARR regiter to generate signal frequency at 17.57 Khz */
	TimerPeriod = (SystemCoreClock / 17570 ) - 1;
	/* Compute CCR1 value to generate a duty cycle at 50% for channel 1 */
	Channel1Pulse = (uint16_t) (((uint32_t) 5 * (TimerPeriod - 1)) / 10);
	/* Compute CCR2 value to generate a duty cycle at 37.5%  for channel 2 */
	Channel2Pulse = (uint16_t) (((uint32_t) 375 * (TimerPeriod - 1)) / 1000);

	/* TIM1 clock enable */
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_TIM1 , ENABLE);

	/* Time Base configuration */
	TIM_TimeBaseStructure.TIM_Prescaler = 0;
	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
	TIM_TimeBaseStructure.TIM_Period = TimerPeriod;
	TIM_TimeBaseStructure.TIM_ClockDivision = 0;
	TIM_TimeBaseStructure.TIM_RepetitionCounter = 0;

	TIM_TimeBaseInit(TIM1, &TIM_TimeBaseStructure);

	/* Channel 1, 2, 3 and 4 Configuration in PWM mode */
	TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM1;
	TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
	TIM_OCInitStructure.TIM_OutputNState = TIM_OutputNState_Enable;
	TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_Low;
	TIM_OCInitStructure.TIM_OCNPolarity = TIM_OCNPolarity_High;
	TIM_OCInitStructure.TIM_OCIdleState = TIM_OCIdleState_Set;
	TIM_OCInitStructure.TIM_OCNIdleState = TIM_OCIdleState_Reset;

	TIM_OCInitStructure.TIM_Pulse = Channel1Pulse;
	TIM_OC1Init(TIM1, &TIM_OCInitStructure);

	TIM_OCInitStructure.TIM_Pulse = Channel2Pulse;
	TIM_OC2Init(TIM1, &TIM_OCInitStructure);

	/* TIM1 counter enable */
	TIM_Cmd(TIM1, ENABLE);

	/* TIM1 Main Output Enable */
	TIM_CtrlPWMOutputs(TIM1, ENABLE);
}

// PWM timer setup example - input sets pwm output
void TIM2_ConfigIRQ()
{
	/* ---------------------------------------------------------------------------
	TIM2 configuration: PWM Input mode

	In this example TIM2 input clock (TIM2CLK) is set to APB1 clock (PCLK1)
	  TIM2CLK = PCLK1
	  => TIM2CLK = HCLK = SystemCoreClock

	External Signal Frequency = TIM2 counter clock / TIM2_CCR2 in Hz.

	External Signal DutyCycle = (TIM2_CCR1*100)/(TIM2_CCR2) in %.

	--------------------------------------------------------------------------- */

	TIM_ICInitStructure.TIM_Channel = TIM_Channel_2;
	TIM_ICInitStructure.TIM_ICPolarity = TIM_ICPolarity_Rising;
	TIM_ICInitStructure.TIM_ICSelection = TIM_ICSelection_DirectTI;
	TIM_ICInitStructure.TIM_ICPrescaler = TIM_ICPSC_DIV1;
	TIM_ICInitStructure.TIM_ICFilter = 0x0;

	TIM_PWMIConfig(TIM2, &TIM_ICInitStructure);

	/* Select the TIM2 Input Trigger: TI2FP2 */
	TIM_SelectInputTrigger(TIM2, TIM_TS_TI2FP2);

	/* Select the slave Mode: Reset Mode */
	TIM_SelectSlaveMode(TIM2, TIM_SlaveMode_Reset);
	TIM_SelectMasterSlaveMode(TIM2,TIM_MasterSlaveMode_Enable);

	/* TIM enable counter */
	TIM_Cmd(TIM2, ENABLE);

	/* Enable the CC2 Interrupt Request */
	TIM_ITConfig(TIM2, TIM_IT_CC2, ENABLE);
}

/**
  * @brief  Configure the TIM2 IRQ Handler.
  */
void TIM2_Config(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;
	NVIC_InitTypeDef NVIC_InitStructure;

	/* TIM2 clock enable */
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, ENABLE);

	/* GPIOA clock enable */
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);

	/* TIM2 chennel2 configuration : PA.01 */
	GPIO_InitStructure.GPIO_Pin   = GPIO_Pin_1;
	GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_AF;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_UP ;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	/* Connect TIM pin to AF2 */
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource1, GPIO_AF_2);

	/* Enable the TIM2 global Interrupt */
	NVIC_InitStructure.NVIC_IRQChannel = TIM2_IRQn;
//	NVIC_InitStructure.NVIC_IRQChannelPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);
}

/**
  * @brief  Configure the TIM3 IRQ Handler.
  */
void TIM3_Int_Config(void)
{
	NVIC_InitTypeDef NVIC_InitStructure;

	/* TIM3 clock enable */
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3, ENABLE);

	/* Enable the TIM3 global Interrupt */
	NVIC_InitStructure.NVIC_IRQChannel = TIM3_IRQn;
	//NVIC_InitStructure.NVIC_IRQChannelPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);
}

// Setup Timer 3 as a 1msec time delay interrupt
void TIM3_Init(void)
{
	uint16_t CCR3_Val,PrescalerValue;

	// TIM Interrupt Configuration
	TIM3_Int_Config();

	/* -----------------------------------------------------------------------
	TIM3 Configuration: Output Compare Timing Mode:

	In this example TIM3 input clock (TIM3CLK) is set to APB1 clock (PCLK1),
	  => TIM3CLK = PCLK1 = SystemCoreClock = 72 MHz

	To get TIM3 counter clock at 6 MHz, the prescaler is computed as follows:
	   Prescaler = (TIM3CLK / TIM3 counter clock) - 1
	   Prescaler = (PCLK1 /6 MHz) - 1

	CC3 update rate = TIM3 counter clock / CCR3_Val = 439.4 Hz
	==> Toggling frequency = 219.7 Hz

	----------------------------------------------------------------------- */

	// Compute the prescaler value
	PrescalerValue = (uint16_t) (SystemCoreClock  / 1000000) - 1;
	//PrescalerValue = 0;

	CCR3_Val = 1000;
	mseccountdown1 = mseccountdown2 = mseccountdown3 = mseccountdown4 = 0;
	mseccounter = 0;

	// Time base configuration
	TIM_TimeBaseStructure.TIM_Period = 65535;
	TIM_TimeBaseStructure.TIM_Prescaler = 0;
	TIM_TimeBaseStructure.TIM_ClockDivision = 0;
	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;

	TIM_TimeBaseInit(TIM3, &TIM_TimeBaseStructure);

	// Prescaler configuration
	TIM_PrescalerConfig(TIM3, PrescalerValue, TIM_PSCReloadMode_Immediate);

	// Output Compare Timing Mode configuration: Channel1
	//TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_Timing;
	//TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;

	// Output Compare Timing Mode configuration: Channel3
	TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
	TIM_OCInitStructure.TIM_Pulse = CCR3_Val;

	TIM_OC3Init(TIM3, &TIM_OCInitStructure);

	TIM_OC3PreloadConfig(TIM3, TIM_OCPreload_Disable);

	// TIM Interrupts enable
	TIM_ITConfig(TIM3, TIM_IT_CC3, ENABLE);

	// TIM3 enable counter
	TIM_Cmd(TIM3, ENABLE);
}

void TIM7_Int_Config(void)
{
	NVIC_InitTypeDef NVIC_InitStructure;

	// Enable the TIM7 global Interrupt
	NVIC_InitStructure.NVIC_IRQChannel = TIM7_IRQn;
	//NVIC_InitStructure.NVIC_IRQChannelPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);
}

// Setup usec delay timer with roll over tally to allow extra long usec timing
// Every 65535 useconds the timer causes an interrupt which keeps a tally of roll overs
void TIM7_Init(void)
{
	uint16_t CCR3_Val,PrescalerValue;

	// Setup TIM7 interrupt handler
	TIM7_Int_Config();

	// TIM7 clock enable
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM7, ENABLE);

	// Compute the prescaler value
	PrescalerValue = (uint16_t) (SystemCoreClock  / 1000000) - 1;

	// Time base configuration
	TIM_TimeBaseStructure.TIM_Period = 65535;
	TIM_TimeBaseStructure.TIM_Prescaler = 0;
	TIM_TimeBaseStructure.TIM_ClockDivision = 0;
	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;

	TIM_TimeBaseInit(TIM7, &TIM_TimeBaseStructure);

	// Prescaler configuration
	TIM_PrescalerConfig(TIM7, PrescalerValue, TIM_PSCReloadMode_Immediate);

	TIM_UpdateRequestConfig(TIM7, TIM_UpdateSource_Regular);

	//TIM7->DIER = TIM_DIER_UIE; // enable the update interrupt

	// TIM Interrupts enable
	TIM_ITConfig(TIM7, TIM_IT_Update, ENABLE);

	// TIM3 enable counter - do this in the start tim7 function
	//TIM_Cmd(TIM7, ENABLE);
}

void I2C1_Init(unsigned long speed)
{
	I2C_InitTypeDef  I2C1_InitStructure;
	GPIO_InitTypeDef GPIO_InitStructure;
    NVIC_InitTypeDef NVIC_InitStructure;

	I2C_DeInit(I2C1);

	// I2C Periph clock enable
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_I2C1, ENABLE);
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);

    // Configure the I2C clock source. The clock is derived from the HSI
    RCC_I2CCLKConfig(RCC_I2C1CLK_HSI);

    GPIO_PinAFConfig(GPIOB, GPIO_PinSource6, GPIO_AF_4);
    GPIO_PinAFConfig(GPIOB, GPIO_PinSource7, GPIO_AF_4);

    //Configure pins: SCL and SDA ------------------
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_7;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_OD;
    GPIO_Init(GPIOB, &GPIO_InitStructure);

    // Init I2C to defaults
    I2C_StructInit(&I2C1_InitStructure);

    I2C1_InitStructure.I2C_AnalogFilter = I2C_AnalogFilter_Enable;
    I2C1_InitStructure.I2C_DigitalFilter = 0x00;
    I2C1_InitStructure.I2C_OwnAddress1 = 0x00;
    I2C1_InitStructure.I2C_Mode = I2C_Mode_I2C;
    I2C1_InitStructure.I2C_Ack = I2C_Ack_Enable;

    /* I2C TIMING Register define when I2C clock source is SYSCLK */
    /* I2C TIMING is calculated in case of the I2C Clock source is the SYSCLK = 72 MHz */
    /* When using SFOX8700 set TIMING to 0x00C4092A to reach 1 MHz speed (Rise time = 26ns, Fall time = 2ns) */
    /* When using slow devices set TIMING to 0xC062121F to reach 100 KHz speed (Rise time = 640ns, Fall time = 20ns) */

    //I2C_InitStructure.I2C_Timing = 0x0x00C4092A;  // 1MHz
    //I2C1_InitStructure.I2C_Timing = 0xC062121F;  // 100KHz
    //I2C1_InitStructure.I2C_Timing = 0x20310A0D;  // 400KHz
    //I2C1_InitStructure.I2C_Timing = 0x50330309; // 400KHz - other source
    I2C1_InitStructure.I2C_Timing = speed;
    I2C1_InitStructure.I2C_AcknowledgedAddress = I2C_AcknowledgedAddress_7bit;

    I2C_Init(I2C1, &I2C1_InitStructure);

	// Configure the I2C interrupt priority
	//NVIC_InitStructure.NVIC_IRQChannel = I2C1_EV_IRQn;
	//NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
	//NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	//NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	//NVIC_Init(&NVIC_InitStructure);

	// Enable I2C
	I2C_Cmd(I2C1, ENABLE);

	// Enable Interrupts
	//I2C_ITConfig(I2C1, I2C_IT_ADDRI | I2C_IT_RXI, ENABLE);
}

void I2C2_Init(unsigned long speed)
{
	I2C_InitTypeDef  I2C2_InitStructure;
	GPIO_InitTypeDef GPIO_InitStructure;
    NVIC_InitTypeDef NVIC_InitStructure;

	I2C_DeInit(I2C2);

	// I2C Periph clock enable
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_I2C2, ENABLE);
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);

    // Configure the I2C clock source. The clock is derived from the HSI
    RCC_I2CCLKConfig(RCC_I2C2CLK_HSI);

    GPIO_PinAFConfig(GPIOA, GPIO_PinSource9, GPIO_AF_4);
    GPIO_PinAFConfig(GPIOA, GPIO_PinSource10, GPIO_AF_4);

    //Configure pins: SCL and SDA ------------------
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_9 | GPIO_Pin_10;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_OD;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    I2C2_InitStructure.I2C_AnalogFilter = I2C_AnalogFilter_Enable;
    I2C2_InitStructure.I2C_DigitalFilter = 0x00;
    I2C2_InitStructure.I2C_OwnAddress1 = 0x00;
    I2C2_InitStructure.I2C_Mode = I2C_Mode_I2C;
    I2C2_InitStructure.I2C_Ack = I2C_Ack_Enable;

    /* I2C TIMING Register define when I2C clock source is SYSCLK */
    /* I2C TIMING is calculated in case of the I2C Clock source is the SYSCLK = 72 MHz */
    /* When using SFOX8700 set TIMING to 0x00C4092A to reach 1 MHz speed (Rise time = 26ns, Fall time = 2ns) */
    /* When using slow devices set TIMING to 0xC062121F to reach 100 KHz speed (Rise time = 640ns, Fall time = 20ns) */

    //I2C_InitStructure.I2C_Timing = 0x0x00C4092A;  // 1MHz
    //I2C1_InitStructure.I2C_Timing = 0xC062121F;  // 100KHz
    //I2C1_InitStructure.I2C_Timing = 0x20310A0D;  // 400KHz
    //I2C1_InitStructure.I2C_Timing = 0x50330309; // 400KHz - other source
    I2C2_InitStructure.I2C_Timing = speed;
    I2C2_InitStructure.I2C_AcknowledgedAddress = I2C_AcknowledgedAddress_7bit;

    I2C_Init(I2C2, &I2C2_InitStructure);

    I2C_Cmd(I2C2, ENABLE);
}

void SPI1_Init(void)
{
    GPIO_InitTypeDef GPIO_InitStructure;
    SPI_InitTypeDef SPI_InitStructure;

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_SPI1, ENABLE);
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA | RCC_AHBPeriph_GPIOE, ENABLE);

    // Configure IO for ST7754 LCD chip
    GPIO_PinAFConfig(GPIOA, GPIO_PinSource5, GPIO_AF_5); // SPI SCK
    GPIO_PinAFConfig(GPIOA, GPIO_PinSource6, GPIO_AF_5); // SPI MISO
    GPIO_PinAFConfig(GPIOA, GPIO_PinSource7, GPIO_AF_5); // SPI MOSI

    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_NOPULL;//GPIO_PuPd_DOWN;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_5 | GPIO_Pin_6 | GPIO_Pin_7;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    SPI_I2S_DeInit(SPI1);
    SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex;
    SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;
    SPI_InitStructure.SPI_CPOL = SPI_CPOL_Low;
    SPI_InitStructure.SPI_CPHA = SPI_CPHA_1Edge;
    SPI_InitStructure.SPI_NSS = SPI_NSS_Soft;
    SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_8;
    SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;
    SPI_InitStructure.SPI_CRCPolynomial = 7;
    SPI_InitStructure.SPI_Mode = SPI_Mode_Master;
    SPI_Init(SPI1, &SPI_InitStructure);

    SPI_RxFIFOThresholdConfig(SPI1, SPI_RxFIFOThreshold_QF);

    SPI_Cmd(SPI1, ENABLE);

    // Ensure RXNE is initially clear
    SPI_ReceiveData8(SPI1);
}

void SPI2_Init(void)
{
    GPIO_InitTypeDef GPIO_InitStructure;
    SPI_InitTypeDef SPI_InitStructure;

    if(1) {
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_SPI2, ENABLE);
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);

    // Configure IO for nRF24L01 SPI SCK, MISO, MOSI
    GPIO_PinAFConfig(GPIOB, GPIO_PinSource13, GPIO_AF_5); // SPI SCK
    GPIO_PinAFConfig(GPIOB, GPIO_PinSource14, GPIO_AF_5); // SPI MISO
    GPIO_PinAFConfig(GPIOB, GPIO_PinSource15, GPIO_AF_5); // SPI MOSI

    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_NOPULL;//GPIO_PuPd_DOWN;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_13 | GPIO_Pin_14 | GPIO_Pin_15;
        GPIO_Init(GPIOB, &GPIO_InitStructure);

	// GPIO SPI2 CS - CE-PB11, CSN-PB12
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_11 | GPIO_Pin_12;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	// OType, Speed and PuPd are the same
	GPIO_Init(GPIOB, &GPIO_InitStructure);

    SPI_I2S_DeInit(SPI2);
    SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex;
    SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;
    SPI_InitStructure.SPI_CPOL = SPI_CPOL_Low;
    SPI_InitStructure.SPI_CPHA = SPI_CPHA_1Edge;
    SPI_InitStructure.SPI_NSS = SPI_NSS_Soft;
    //SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_8; // 4mbit rate
    SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_16;  // 2mbit rate
    SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;
    SPI_InitStructure.SPI_CRCPolynomial = 7;
    SPI_InitStructure.SPI_Mode = SPI_Mode_Master;
    SPI_Init(SPI2, &SPI_InitStructure);

    SPI_RxFIFOThresholdConfig(SPI2, SPI_RxFIFOThreshold_QF);

    //printf("\n\rSPI_BaudRatePrescaler_8=%d\n\r",SPI_BaudRatePrescaler_8);

    SPI_Cmd(SPI2, ENABLE);

    // Ensure RXNE is initially clear
    SPI_ReceiveData8(SPI2);
	}
	// Bit bang SPI function
	else {
	    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);

		// GPIO SPI2 CS - CE-PB11, CSN-PB12, SCK2-PB13, MOSI2-PB15
		GPIO_InitStructure.GPIO_Pin = GPIO_Pin_11 | GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_15;
		GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
		// OType, Speed and PuPd are the same
		GPIO_Init(GPIOB, &GPIO_InitStructure);

		// GPIO SPI2 Input - MISO2-PB14
		GPIO_InitStructure.GPIO_Pin = GPIO_Pin_14;
		GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
		// OType, Speed and PuPd are the same
		GPIO_Init(GPIOB, &GPIO_InitStructure);

		MOSI2HIGH;
		SCK2LOW;
		}
}

uint8_t SPI1_transfer_byte(uint8_t byte)
{
	while (SPI_I2S_GetFlagStatus(SPI1, SPI_I2S_FLAG_TXE) == RESET);
	// Send a Byte through the SPI peripheral
	SPI_SendData8(SPI1, byte);
	while (SPI_I2S_GetFlagStatus(SPI1, SPI_I2S_FLAG_RXNE) == RESET);
	return (uint8_t)SPI_ReceiveData8(SPI1);
}

// Bit bang version - pretty slow
uint8_t SPI2_transfer_byte_old(uint8_t outbyte)
{
	unsigned char inbyte,mask=0x80;
	int i;

	inbyte = 0;
	for(i=0; i<8; i++) {
		if(outbyte&mask) MOSI2HIGH;
		else MOSI2LOW;
		delay_us(1);
		SCK2HIGH;
		if(GPIO_ReadInputDataBit(GPIOB,GPIO_Pin_14)) inbyte |= mask;
		delay_us(1);
		SCK2LOW;
		//delay_us(2);
		mask = mask >> 1;
		}
	MOSI2HIGH;
	return inbyte;
}

uint8_t SPI2_transfer_byte(uint8_t byte)
{
	while (SPI_I2S_GetFlagStatus(SPI2, SPI_I2S_FLAG_TXE) == RESET);
	// Send a Byte through the SPI peripheral
	SPI_SendData8(SPI2, byte);
	while (SPI_I2S_GetFlagStatus(SPI2, SPI_I2S_FLAG_RXNE) == RESET);
	return (uint8_t)SPI_ReceiveData8(SPI2);
}

/***********************************************
 *
 *  ISRs
 *
 ***********************************************/
/**
  * @brief  This function handles TIM2 global interrupt request.
  */
void TIM2_IRQHandler(void)
{
	/* Clear TIM2 Capture compare interrupt pending bit */
	//TIM_ClearITPendingBit(TIM2, TIM_IT_CC2);
	TIM2->SR = (uint16_t)~TIM_IT_CC2;

	/* Get the Input Capture value */
	//IC2Value = TIM_GetCapture2(TIM2);
	IC2Value = TIM2->CCR2;
}

// Master 1 msec interrupt handler for delay countdown
void TIM3_IRQHandler(void)
{
    if (TIM_GetITStatus(TIM3, TIM_IT_CC3) != RESET)  {
		TIM_ClearITPendingBit(TIM3, TIM_IT_CC3);
		//LED2HIGH;
		if(mseccountdown1) mseccountdown1--;
		if(mseccountdown2) mseccountdown2--;
		if(mseccountdown3) mseccountdown3--;
		if(mseccountdown4) mseccountdown4--;
		if(bmp180countdown) bmp180countdown--;
		mseccounter++;
		TIM_SetCounter(TIM3, 0);
		//LED2LOW;
    	}
}

// Keep track of usectimer roll overs
void TIM7_IRQHandler(void)
{
    if(TIM_GetITStatus(TIM7, TIM_IT_Update) != RESET)  {
		TIM_ClearITPendingBit(TIM7, TIM_IT_Update);
		usectimertally++;
    	}
}

// Interrupt handler for USART3  TX and RX buffer size set by USART3_TX_BUFSIZE and USART3_RX_BUFSIZE
void USART2_IRQHandler(void)
{
	// Check receive interrupt
    if (USART_GetITStatus(USART2, USART_IT_RXNE) != RESET)  {
    	USART_ClearITPendingBit(USART2, USART_IT_RXNE);
    	//LED1LOW;
		rx2buffer[rx2headptr++] = (uint8_t)(USART2->RDR & (uint16_t)0x00FF);
		if(rx2headptr==USART2_RX_BUFSIZE) rx2headptr = 0;
		if(rx2headptr==rx2tailptr) {
			rx2tailptr++;
			if(rx2tailptr==USART2_RX_BUFSIZE) rx2tailptr = 0;
			}
    	}
    // Check transmit interrupt
    if (USART_GetITStatus(USART2, USART_IT_TXE) != RESET)  {
    	USART_ClearITPendingBit(USART2, USART_IT_TXE);
    	//LED2LOW;
		USART2->TDR = tx2buffer[tx2tailptr++];
		if(tx2tailptr==USART2_TX_BUFSIZE) tx2tailptr = 0;
		// Turn off interrupt is not more data to transmit
		if(tx2tailptr==tx2headptr)  USART_ITConfig(USART2, USART_IT_TXE, DISABLE);
    	}
}

/***********************************************
 *  I2C Functions
 ***********************************************/
#define INIT_TIMEOUT i2c_Timeout = I2C_LONG_TIMEOUT
#define CHECK_TIMEOUT if((i2c_Timeout--) == 0) return I2C_TIMEOUT_UserCallback()

uint8_t I2C_TIMEOUT_UserCallback(void)
{
	i2ctimedout = YES;
	return i2ctimedout;
}

//  Write to the specified number of bytes to the I2C device
uint8_t I2C_WriteBytes(I2C_TypeDef* I2Cx,uint8_t twiaddress,uint8_t nbytes)
{
	int i=0;
	i2ctimedout = NO;

	// Test on BUSY Flag
	INIT_TIMEOUT;
	while(I2C_GetFlagStatus(I2Cx, I2C_ISR_BUSY) != RESET) {CHECK_TIMEOUT;}

	// Configure slave address, nbytes, reload, end mode and start or stop generation
	I2C_TransferHandling(I2Cx, twiaddress<<1, nbytes, I2C_AutoEnd_Mode, I2C_Generate_Start_Write);

	while(nbytes--) {
		// Wait until TXIS flag is set
		INIT_TIMEOUT;
		while(I2C_GetFlagStatus(I2Cx, I2C_ISR_TXIS) == RESET)  {CHECK_TIMEOUT;}

		// Send Register address
		I2C_SendData(I2Cx, (uint8_t)messageBuf[i++]);
		}

	// Wait until STOPF flag is set
	INIT_TIMEOUT;
	while(I2C_GetFlagStatus(I2Cx, I2C_ISR_STOPF) == RESET) {CHECK_TIMEOUT;}

	// Clear STOPF flag
	I2C_ClearFlag(I2Cx, I2C_ICR_STOPCF);

	return I2C_OK;
}

// Read the specified number of bytes from the I2C device.
// I2Cx - Select I2C1 or I2C2
// twiaddress - I2C address (must shift left one bit
// nbytes - Number of bytes to write
uint8_t I2C_ReadBytes(I2C_TypeDef* I2Cx,uint8_t twiaddress,uint8_t nbytes)
{
	int i=0;
	i2ctimedout = NO;

	// Test on BUSY Flag
	INIT_TIMEOUT;
	while(I2C_GetFlagStatus(I2Cx, I2C_ISR_BUSY) != RESET) {CHECK_TIMEOUT;}

	// Configure slave address, nbytes, reload, end mode and start or stop generation
	I2C_TransferHandling(I2Cx, twiaddress<<1, nbytes, I2C_AutoEnd_Mode, I2C_Generate_Start_Read);

	while(nbytes--) {
		// Wait until RXNE flag is set
		INIT_TIMEOUT;
		while(I2C_GetFlagStatus(I2Cx, I2C_ISR_RXNE) == RESET) {CHECK_TIMEOUT;}

		// Read data from RXDR
		messageBuf[i++]= I2C_ReceiveData(I2Cx);
		}

	// Wait until STOPF flag is set
	INIT_TIMEOUT;
	while(I2C_GetFlagStatus(I2Cx, I2C_ISR_STOPF) == RESET) {CHECK_TIMEOUT;}

	// Clear STOPF flag
	I2C_ClearFlag(I2Cx, I2C_ICR_STOPCF);

	// return a Reg value
	return messageBuf[0];
}

// Read the specified register from the I2C device using a repeated start - combined
// I2Cx - Select I2C1 or I2C2
// twiaddress - I2C address (must shift left one bit
// RegId - specifies the device register to be read.
// nbytes - number of bytes to read
uint8_t I2C_WriteReadBytes(I2C_TypeDef* I2Cx,uint8_t twiaddress,uint8_t RegId,uint8_t nbytes)
{
	int i=0;
	i2ctimedout = NO;

	// Test on BUSY Flag
	INIT_TIMEOUT;
	while(I2C_GetFlagStatus(I2Cx, I2C_ISR_BUSY) != RESET) {CHECK_TIMEOUT;}

	// Configure slave address, nbytes, reload, end mode and start or stop generation
	I2C_TransferHandling(I2Cx, twiaddress<<1, 1, I2C_SoftEnd_Mode, I2C_Generate_Start_Write);

	// Wait until TXIS flag is set
	INIT_TIMEOUT;
	while(I2C_GetFlagStatus(I2Cx, I2C_ISR_TXIS) == RESET) {CHECK_TIMEOUT;}

	// Send Register address
	I2C_SendData(I2Cx, (uint8_t)RegId);

	// NEED TO GENERATE RESTART CONDITION HERE
	//I2C_GenerateSTART(I2C1, ENABLE);
	//while(I2C_GetFlagStatus(I2C1,I2C_FLAG_BUSY));

	// Wait until TC flag is set
	INIT_TIMEOUT;
	while(I2C_GetFlagStatus(I2Cx, I2C_ISR_TC) == RESET) {CHECK_TIMEOUT;}

	// Configure slave address, nbytes, reload, end mode and start or stop generation
	I2C_TransferHandling(I2Cx, twiaddress<<1, nbytes, I2C_AutoEnd_Mode, I2C_Generate_Start_Read);
	//I2C_GenerateSTART(I2C1, ENABLE);

	while(nbytes--) {
		// Wait until RXNE flag is set
		INIT_TIMEOUT;
		while(I2C_GetFlagStatus(I2Cx, I2C_ISR_RXNE) == RESET) {CHECK_TIMEOUT;}

		// Read data from RXDR
		messageBuf[i++]= I2C_ReceiveData(I2Cx);
		}

	// Wait until STOPF flag is set
	INIT_TIMEOUT;
	while(I2C_GetFlagStatus(I2Cx, I2C_ISR_STOPF) == RESET) {CHECK_TIMEOUT;}

	// Clear STOPF flag
	I2C_ClearFlag(I2Cx, I2C_ICR_STOPCF);

	// return a Reg value
	return messageBuf[0];
}

/***********************************************
 *  Serial IO Functions for USB and USART3
 ***********************************************/
int GetCharnw2(void)
{
	int intchar;

	// Uncomment this call if rx buffer is experiencing an overrun condition
	//if(USART_GetFlagStatus(USART3, USART_FLAG_ORE) == SET) {
	//	USART_ClearFlag(USART3, USART_FLAG_ORE);
	//	}

	if(rx2headptr!=rx2tailptr) {
		intchar = rx2buffer[rx2tailptr++];
		if(rx2tailptr==USART2_RX_BUFSIZE) rx2tailptr = 0;
		return intchar;
		}
	else return -1;
}

unsigned char GetChar2(void)
{
	int intchar;

	while((intchar=GetCharnw3())==-1) ;
	return (unsigned char) intchar;
}

void PutChar2(unsigned char outchar)
{
	if(tx2headptr==tx2tailptr) {
		tx2buffer[tx2headptr++] = outchar;
		if(tx2headptr==USART2_TX_BUFSIZE) tx2headptr = 0;
		USART_ITConfig(USART2, USART_IT_TXE, ENABLE);
		}
	else {
		tx2buffer[tx2headptr++] = outchar;
		if(tx2headptr==USART2_TX_BUFSIZE) tx2headptr = 0;
		if(tx2tailptr==tx2headptr) {
			tx2tailptr++;
			if(tx2tailptr==USART2_TX_BUFSIZE) tx2tailptr = 0;
			}
		}
}

void Puts2(unsigned char *outstrg)
{
	int i;

	for(i=0; i<strlen(outstrg); i++) PutChar2(outstrg[i]);
}

// Formated output for USART3
int printf2(const char *fmt, ...)
{
	va_list args;
	int i,j;
	char printbuffer[150];

	va_start (args, fmt);
	i = vsnprintf (printbuffer, 150, fmt, args);
	va_end (args);

	// Print the string
	j = 0;
	while(printbuffer[j]!=0) PutChar2(printbuffer[j++]);

	return i;
}

int scanf2(const char *fmt, ...)
{
	char inchar;
	va_list args;
	int i,ptr;
	char scanbuffer[120];

	ptr = 0;
	while((inchar=GetChar2())!=CR) {
        scanbuffer[ptr++] = inchar;
		//putchar(inchar);
		}
	scanbuffer[ptr] = 0;

	va_start (args, fmt);
	i = vsscanf(scanbuffer, fmt, args);
	va_end (args);

	return i;
}
/***********************************************
 *  Utility Functions
 ***********************************************/
void TIM7_Start(void)
{
	TIM_SetCounter(TIM7, 0);
	TIM_Cmd(TIM7,ENABLE);
	usectimertally = 0;
}

void TIM7_Stop(void)
{
	TIM_Cmd(TIM7,DISABLE);
}

void delay_us(unsigned int ult)
{
	TIM7_Start();

	while(TIM_GetCounter(TIM7)<ult) ;
	TIM7_Stop();
}

void delay_ms(unsigned int ult)
{
	mseccountdown1 = ult;
	while(mseccountdown1) ;
}

void Toggle_Leds(int grncnt)
{
	if(!mseccountdown2) {
		mseccountdown2 = grncnt;
		if(grnflip) {
			LED1LOW;
			grnflip = 0;
			}
		else {
			LED1HIGH;
			grnflip = 1;
			}
		}
}

/***********************************************
 *
 *  Work Functions
 *
 ***********************************************/
void PicoM_Init(void)
{
	LED_Init();   // Leds on PC8 and PC9
	PortA_Init();

	TIM3_Init();  // Setup 1msec count down timer
	TIM7_Init(); // Setup usec count down timer

	// Initialize I2C at Fast Speed to ensure correct handling of default I2C devices
	//I2C1_Init(I2C100KHZ);
	//I2C2_Init(I2C100KHZ);
	I2C1_Init(I2C400KHZ);
	I2C2_Init(I2C400KHZ);

	SPI2_Init();

	// Pin PA8 is connected to VBUS with a pull down - if USB cable connected
	// the input is high else low
	VBUS_Monitor_Init();

	LED0OFF;

	//USART2_Init(38400);  // Setup USART2
	rxtailptr = rxheadptr = 0;

	// See what I2C devices there are
	Enumerate_TWI_Devices(QUIET);

	// Setup MPU9150 if enabled
	if(mpu9150state) MPU9150_Init();

	// Setup BMP180 if enabled
	if(BMPstate)  {
		Load_BMP180_Parameters();
		Init_BMP180_Readings();
		}

	// Check for RF24L01 and initialize if present
	delay_ms(12);  // Let RF24L01 come out of power on reset
	rf_device_class = DEVICE_CLASS;
	device_id = PICOM_DEVICE_ID;
	rf_payload_size = PAYLOAD_SIZE;
	channel_no = CHANNEL_NO;
	rf24L01_Init(RFRECEIVER);
 	//rf24L01_Init(RFSENDER);
	if(rf_state==RF_STATE_INITIALIZED_TX)  rf_mode = RF_STREAM;  // Enable RF stream mode by default

#ifdef SSD1306OLED
	delay_ms(40);
	ssd1306_Init();
	ssd1306_clear(0);
	#ifdef DISPLAYSENSORDATA
		updatedisplay = ACTIVESTATE;
		updaterate = DISPLAYPDATERATE;
	#else
		updatedisplay = INACTIVE;
	#endif
#endif
	//  Initialize all 15 9DOF finger sensors if PicoM305 - FXOS8700 has 1000msec boot time
#ifdef PICOM305TARGET
	delay_ms(1000);
	FXOS8700_Init_All(QUIET);
	// Boost I2C to maximum rate for fast finger orientation capture
	I2C1_Init(I2C1000KHZ);
	I2C2_Init(I2C1000KHZ);
#endif

#ifdef DISPLAYSENSORDATA
	memsstate = ACTIVESTATE;
#else
	memsstate = INACTIVE;  // Enable once RF24L01 is interrupt driven
#endif
}

// More extension initialization
void STMF303_Init(void)
{
	 // Setup All ports
	GPIO_InitTypeDef  GPIO_InitStructure;

	/* Enable the GPIO_LED Clock */
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);

	/* Configure the GPIOA output pins */
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_14 | GPIO_Pin_13 | GPIO_Pin_12 | GPIO_Pin_11 |
								  GPIO_Pin_8 | GPIO_Pin_3;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	/* Configure the GPIOA input pins */
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_2;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
	//GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	/* Configure the GPIOB output pins */
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);

	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_7 | GPIO_Pin_5 | GPIO_Pin_4 | GPIO_Pin_3 | GPIO_Pin_0;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOB, &GPIO_InitStructure);

	/* Configure the GPIOB input pins */
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_1;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
	//GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOB, &GPIO_InitStructure);

	/* Configure the GPIOF pins */
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOF, ENABLE);

	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1 | GPIO_Pin_0;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOF, &GPIO_InitStructure);

	LED_Init();   // Leds on PC8 and PC9

	TIM3_Init();  // Setup 1msec count down timer
	TIM7_Init(); // Setup usec count down timer

	LED0OFF;

	USART3_Init(38400);  // Setup USART3
}

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/

/*******************************************************************************
* Function Name  : main.
* Description    : Main routine.
* Input          : None.
* Output         : None.
* Return         : None.
*******************************************************************************/
extern void PutChar(unsigned char inchar);

/* Private variables ---------------------------------------------------------*/
  RCC_ClocksTypeDef RCC_Clocks;
  uint32_t TimingDelay1 = 0,TimingDelay2=0,TimingDelay3=0,TimingDelay4=0;

/* Private function prototypes -----------------------------------------------*/
void TimingDelay_Decrement(void);
void Delay(uint32_t nTime);

/* Private functions ---------------------------------------------------------*/

/**
  * @brief  Decrements the TimingDelay variable.
  * @param  None
  * @retval None
  */
void TimingDelay_Decrement(void)
{
  if (TimingDelay1 != 0x00) TimingDelay1--;
  if (TimingDelay2 != 0x00) TimingDelay2--;
  if (TimingDelay3 != 0x00) TimingDelay3--;
  if (TimingDelay4 != 0x00) TimingDelay4--;
}

/**
  * @brief  Inserts a delay time.
  * @param  nTime: specifies the delay time length, in 10 ms.
  * @retval None
  */
void Delay(uint32_t nTime)
{
  TimingDelay1 = nTime;
  printf("\n\r");
  while(TimingDelay1 != 0)  printf("%d   \r",TimingDelay1);;
}

int __errno(void)
 {
	 int i=0;

	 i += 1;
	 return i;
 }

unsigned char GetCharnw(void)
{
	char inchar;
	int tempptr;

	if(rxheadptr==rxtailptr)  return 0;
	inchar = rxbuf[rxtailptr];
	rxtailptr++;
	if(rxtailptr==RXBUFSIZE) rxtailptr = 0;
	return inchar;
}

unsigned char GetChar(void)
{
	char inchar;

	while(!(inchar=GetCharnw())) ;
	return inchar;
}

void Puts(char *inchar)
{
	int i;

	for(i=0; i<strlen(inchar); i++) PutChar(inchar[i]);
}


int printf (const char *fmt, ...)
{
	va_list args;
	int i,j;
	char printbuffer[250];

	va_start (args, fmt);
	i = vsnprintf (printbuffer, 250, fmt, args);
	va_end (args);

	// Print the string
	j = 0;
	while(printbuffer[j]!=0) PutChar(printbuffer[j++]);

	return i;
}

int scanf(const char *fmt, ...)
{
	char inchar;
	va_list args;
	int i,ptr;
	char scanbuffer[120];

	//printf("\n\rFormat: %s  args=%d",fmt,args);

	ptr = 0;
	while((inchar=GetChar())!=CR) {
        scanbuffer[ptr++] = inchar;
		//putchar(inchar);
		}
	scanbuffer[ptr] = 0;

	va_start (args, fmt);
	i = vsscanf(scanbuffer, fmt, args);
	va_end (args);

	return i;
}

void Printf_RCC_Clocks(void)
{
	RCC_ClocksTypeDef RCC_Clocks;
	RCC_GetClocksFreq(&RCC_Clocks);

	printf("\n\rRCC Clocks\n\r");
	printf("RCC_Clocks->SYSCLK_Frequency = %ld\n\r",RCC_Clocks.SYSCLK_Frequency);
	//printf("RCC_Clocks->USBCLK_Frequency = %ld\n\r>",RCC_Clocks.USBCLK_Frequency);
}

#ifdef ST7735LCD
void ST7735_Rotation_Test(void)
{
	int rotationrate = 750;

	printf("\n\rCurrent rotation = %d\n\r",ST7735_getRotation());
	printf("Rotate 0 degree\n\r");
	ST7735_fillScreen(colorsRGB24toRGB565(255,255,255));
	ST7735_setRotation(ROTATE0);
	Signon();
	delay_ms(rotationrate);
	printf("Rotate 90 degree\n\r");
	ST7735_fillScreen(colorsRGB24toRGB565(255,255,255));
	ST7735_setRotation(ROTATE90);
	Signon();
	delay_ms(rotationrate);
	printf("Rotate 180 degree\n\r");
	ST7735_fillScreen(colorsRGB24toRGB565(255,255,255));
	ST7735_setRotation(ROTATE180);
	Signon();
	delay_ms(rotationrate);
	printf("Rotate 270 degree\n\r");
	ST7735_fillScreen(colorsRGB24toRGB565(255,255,255));
	ST7735_setRotation(ROTATE270);
	Signon();
	delay_ms(rotationrate);
}

void ST7735_Test(void)
{
	volatile char inchar,flip=0,displaymode='h';
	volatile int i=0;

	printf("\n\rTest ST7735 LCD Display - e to exit\n\r");
	ST7735_InitR();
	ST7735_fillScreen(colorsRGB24toRGB565(255,255,255));

	while ((inchar=getnwchar())!='e') {
		if(inchar=='h') {
			displaymode = 'h';
			ST7735_fillScreen(colorsRGB24toRGB565(255,255,255));
			}
		if(inchar=='r') {
			displaymode = 'r';
			ST7735_fillScreen(colorsRGB24toRGB565(255,255,255));
			}
		if(displaymode=='h') {
			ST7735_drawString(1, 1+i, "PicoAL 10DOF V1.00",colorsRGB24toRGB565(0,0,0), 1);
			ST7735_drawString(1, 20+i,"Wireless",colorsRGB24toRGB565(255,0,0), 2);
			Delaymsec(10);
			ST7735_fillRect(1, 1+i, 110, 10,colorsRGB24toRGB565(255,255,255));
			ST7735_fillRect(1, 20+i, 100, 20,colorsRGB24toRGB565(255,255,255));
			//ST7735_drawString(0, 0+i,"PicoAL 10DOF V1.00",colorsRGB24toRGB565(255,255,255), 1);
			//ST7735_drawString(0, 20+i,"Wireless",colorsRGB24toRGB565(255,255,255), 2);
			if(flip==0) {
				i++;
				if(i>75) flip = 1;
				}
			else {
				i--;
				if(i==0) flip = 0;
				}
			}
		else ST7735_Rotation_Test();
		}
	if(displaymode=='h') {
		ST7735_drawString(1, 1+i,"PicoAL 10DOF V1.01",colorsRGB24toRGB565(0,0,0), 1);
		ST7735_drawString(1, 20+i,"Wireless",colorsRGB24toRGB565(255,0,0), 2);
		}
	else {
		printf("Rotate 0 degree\n\r");
		ST7735_fillScreen(colorsRGB24toRGB565(255,255,255));
		ST7735_setRotation(192);
		Signon();;
		}
	printf("\n\r-");
}
#endif

void Check_RF_Connection(void)
{
	if(rf_state==RF_STATE_INITIALIZED_RX)  {
		// true if message came in
		if(RF24L01_Process_Commands()) {
			rf_inmessage = true;
			rf_cnt = mseccounter;
#ifdef  ST7735LCD
			ST7735_drawString(1,149,"RF Message Received",colorsRGB24toRGB565(0,0,0),1);
#endif
			}
		}

	// Check if inbound rf message occurred
	if(rf_inmessage) {
		if(mseccounter>rf_cnt+1500) {
#ifdef  ST7735LCD
			ST7735_fillRect(1, 149, 126, 10,colorsRGB24toRGB565(255,255,255));
#endif
			rf_inmessage = false;
			}
		}
}

void Signon(void)
{
#ifdef  PICOM04TARGET
	printf("\n\rPicoM04 10DOF Wireless Sensor Node V1.00\n\r");
#endif
#ifdef  PICOM07TARGET
	printf("\n\rPicoM07 10DOF Wireless Sensor Node V1.00\n\r");
#endif
#ifdef  PICOM30TARGET
	printf("\n\rPaellaM30 9DOF Raspberry Pi V1.00\n\r");
#ifdef SSD1306OLED
	ssd1306_printf(0,0,"PicoM 10DOF V1.0");
#endif
#endif
	printf("Copyright 2014 SOC Robotics, Inc.\n\r");
	Display_RF_Status();
	printf("\n\r$");
}

void ESC_Processor(void)
{
	char inchar;

	printf("ESC");
	PutChar(inchar = GetChar());
	PutChar(GetChar());  // Get CR
	if(inchar=='i') {
#ifdef  PICOM04TARGET
#ifdef  ST7735LCD
		printf("\n\rPicoM04L 10DOF V1.00\n\r$");
#else
		printf("\n\rPicoM04 10DOF V1.00\n\r$");
#endif
#endif
#ifdef PICOM07TARGET
#ifdef  ST7735LCD
		printf("\n\rPicoM07L 10DOF V1.00\n\r$");
#else
		printf("\n\rPicoM07 10DOF V1.00\n\r$");
#endif
#endif
#ifdef PICOM30TARGET
#ifdef  ST7735LCD
		printf("\n\rPaellaM30L 10DOF V1.00\n\r$");
#else
		printf("\n\rPaellaM30 10DOF V1.00\n\r$");
#endif
#endif
		}
}

void USB_Connect(void)
{
	Set_USBClock();
	USB_Interrupts_Config();
	USB_Init();
	usbconnected = true;
}

// TBD - disable USB
void USB_Disconnect(void)
{
	USB_Cable_Config(DISABLE);
	usbconnected = false;
}

// **********************************************************
//   Main work starts here
// **********************************************************
int main(void)
{
	char inchar,inchar2,*charptr,*charptr2;
	unsigned char j;
	int i;
	volatile float a,b,angle,angleinc,radius;
	volatile char displaymode;
	volatile int mxmin,mxmax,mymin,mymax,mzmin,mzmax,mxfirst,mxaverage;
	volatile long mxtally;
	volatile float magvector,heading,inclination=LOCALINCLINATION,declination=LOCALDECLINATION,bmptemp,mputemp;
	volatile float elapsedtime;
	unsigned long starttime,endtime;

	union {
	    unsigned char tempbuf[2];
		unsigned short int outint;
		} u;

	Set_System();

	// Initialize all basic peripherals
	PicoM_Init();
	//STMF303_Init();

#ifdef CHECKUSBPA8
	// Only start USB if USB cable connected
	if(GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_8)) {
		//USB_Connect();
		usbconnected = true;
		}
	else {
		usbconnected = false;
		}
#else
	//USB_Connect();
	usbconnected = true;
#endif

	//USB_OTG_dev.regs.DREGS->DSTS (bit 0)  To see if USB cable connect3ed

	// SysTick end of count event each 10ms
	RCC_GetClocksFreq(&RCC_Clocks);
	SysTick_Config(RCC_Clocks.HCLK_Frequency / 100);

RESTART:	// Restart with USB still running so link isn't broken

	Signon();

	Enumerate_TWI_Devices(VERBOSE);  // See what sensors and or devices are attached
	printf("\n\r$");

	//  Loop here listening for commands coming in on USB line or RF24L01 wireless.
	//  This is the top level command loop
   while (1)  {

#ifdef CHECKUSBPA8
	   // Check status of USB connection
	   if(GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_8)) {
		   //if(usbconnected==false) USB_Connect();
	   	   }
	   else if(usbconnected) {
		   USB_Disconnect();
	   	   }
#endif
		// Check for commands coming in on the RF24L01 link and process them
		// TBD - convert to interrupt function
		Check_RF_Connection();

		// Update MEMs sensor data - keep tracking slow update sensors like BMP180 barometer
		if(memsstate==ACTIVESTATE) MEMS_Sensor_Update();

		// Update display if present -
		if(updatedisplay==ACTIVESTATE) Update_Display();

		inchar = GetCharnw();
		if(inchar) {
			if((inchar!=ESC)&&(inchar!=0x05)) PutChar(inchar);

			switch (inchar) {
				case ESC:
					ESC_Processor();
					break;
				case 'r':
				    printf("\n\rRestart PaellaM");
					delay_ms(40);
					//I2C1_Init(I2C400KHZ);
					goto RESTART;
					break;
				case 's':
					Signon();
					printf("\n\rPin PA=%d\n\r$",GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_8));
					//printf("\n\rstack pointer = %ld  %x\n\r",__get_MSP(),__get_MSP());
					//printf("\n\rprocess stack pointer = %ld  %x\n\r",__get_PSP(),__get_PSP());
					break;
				case 'l':
					SSD1306_Test();
					break;
				case 'c':
					Printf_RCC_Clocks();
					break;
				// Switch both I2C peripherals to a new clock rate
				case 'x':
					PutChar(inchar2=GetChar());
					if(inchar2=='1') {
						printf("\n\rReset I2C clock to 100KHz");
						I2C1_Init(I2C100KHZ);
						I2C2_Init(I2C100KHZ);
						}
					else if(inchar2=='4') {
						printf("\n\rReset I2C clock to 400KHz");
						I2C1_Init(I2C400KHZ);
						I2C2_Init(I2C400KHZ);
						}
					else if(inchar2=='f') {
						printf("\n\rReset I2C clock to 800KHz");
						I2C1_Init(I2C1000KHZ);
						I2C2_Init(I2C1000KHZ);
						}
					printf("\n\r-");
					break;
				case 'i':
					inchar2 = GetChar();
					printf("\n\rTest I2C with PCA9545\n\r");
					if(inchar2=='w') {
						messageBuf[0] = PCA9545_CHANNEL_0;
						if(!I2C_WriteBytes(I2C1,PCA9545_SLAVE_ADDRESS,1)) {
							printf("write timed out ic2timedout\n\r-");
							}
						else printf("Write byte\n\r-");
						}
					else {
						messageBuf[0] = 0;
						if(!I2C_ReadBytes(I2C1, PCA9545_SLAVE_ADDRESS,1)) {
							printf("read timed out ic2timedout\n\r-");
							}
						else printf("MessageBuf[0]=%x\n\r-",messageBuf[0]);
						}
				    if(1) {
						messageBuf[0] = PCA9545_CHANNEL_1;
						I2C_WriteBytes(I2C1,PCA9545_SLAVE_ADDRESS,1);
						messageBuf[0] = 0;
						I2C_ReadBytes(I2C1, PCA9545_SLAVE_ADDRESS,1);
						printf("MessageBuf[1]=%x\n\r",messageBuf[0]);
						messageBuf[0] = PCA9545_CHANNEL_2;
						I2C_WriteBytes(I2C1,PCA9545_SLAVE_ADDRESS,1);
						messageBuf[0] = 0;
						I2C_ReadBytes(I2C1, PCA9545_SLAVE_ADDRESS,1);
						printf("MessageBuf[2]=%x\n\r",messageBuf[0]);
						messageBuf[0] = PCA9545_CHANNEL_3;
						I2C_WriteBytes(I2C1,PCA9545_SLAVE_ADDRESS,1);
						messageBuf[0] = 0;
						I2C_ReadBytes(I2C1, PCA9545_SLAVE_ADDRESS,1);
						printf("MessageBuf[3]=%x\n\r-",messageBuf[0]);
						}
				    break;
				case 'd':
					printf("\n\rStart 2000 msec delay");
					delay_ms(2000);
					printf(" - end\n\r");
					printf("Start 2,000,000 usec delay");
					for(i=0; i<200; i++) {
						delay_us(10000);
						}
					printf(" - end\n\r$");
					break;
#ifdef ST7735LCD
				case 'l':
					ST7735_Test();
				break;
#endif
				case 'm':
					MPU9150_Test();
					break;
#ifdef FXOS8700SENSOR
				case 'f':
					PutChar(inchar2 = GetChar());
					if(inchar2=='a') {
						FXOS8700_Init_All(VERBOSE);

						// Read all 15 finger sensors
						TIM7_Start();
						FXOS8700_Display_All(VERBOSE);
						TIM7_Stop();
						printf("Data acquisition and conversion time=%ld usec\n\r",ElapsedTimeusec());

						TIM7_Start();
						FXOS8700_Display_All(QUIET);
						TIM7_Stop();
						printf("Data acquisition time=%ld usec",ElapsedTimeusec());
						}
					else if(inchar2=='w'){
						FXOS8700_Display_All(VERBOSE);
						}
					else if(inchar2=='q') {
						printf("\n\r");
						for(i=0; i<4; i++) {
							printf("f%d: d= -0.932 -0.141  0.336   p= 0.918 -0.473 -0.201   m=-0.570 -0.812 -0.090\n\r",i);
							}
						printf("f4: d= -0.932 -0.141  0.336   p= 0.918 -0.473 -0.201   m=-0.570 -0.812 -0.090");
						}
					else if(inchar2=='r') {
						while((inchar2=GetCharnw())!='e') {
							printf("\n\r");
							for(i=0; i<4; i++) {
								printf("f%d: d= -0.932 -0.141  0.336   p= 0.918 -0.473 -0.201   m=-0.570 -0.812 -0.090\n\r",i);
								}
							printf("f4: d= -0.932 -0.141  0.336   p= 0.918 -0.473 -0.201   m=-0.570 -0.812 -0.090");
							}
						}
					printf("\n\r$");
					break;
#endif
				case 't':
					MPU9150_Read_Temp();
					printf("\n\rMPU9150 T=%s ",ftoa(imu.tempf,2,7));
					bmptemp = CalculateBMP180Temperature(Read_BMP180_Temperature());
					printf(" BMP180 T=%s ",ftoa(bmptemp,2,7));
					printf("\n\rMPU9150 cT=%s ",ftoa(imu.tempf+MPU9150TEMPOFFSET,2,7));
					printf(" BMP180 cT=%s\n\r$",ftoa(bmptemp+BMP180TEMPOFFSET,2,7));
					break;
				case 'b':
					//if(BMPstate) BMP180_PrintData();
					if(BMPstate) {
						Load_BMP180_Parameters();
						DisplayBMP180Brief('a');
						}
					else printf("b\n\rNo barometer installed\n\r$");
					break;
				case 'e':
					Enumerate_TWI_Devices(VERBOSE);  // Enumerate TWI devices
					printf("\n\r$");
					//Read_SmartLCD_Switches_Accelerometer(SMARTLCDTWIADDRESS);
					//printf("\n\rswitch = %x %d %d %d %d",lcdswitch,lcdswitchstate[0],lcdswitchstate[1],lcdswitchstate[2],lcdswitchstate[3]);
					//Delaymsec(10);
					//printf("\n\rswitch2=%x %d",Read_SmartLCD_Switches(SMARTLCDTWIADDRESS),lcdrotary);
					break;
				case '6':
					printf("\n\rTest FXOS8700\n\r");
					messageBuf[0] = PCA9545_CHANNEL_3;
					I2C_WriteBytes(I2C1,PCA9545_SLAVE_ADDRESS,1);

					printf("\n\rFXOS address=%x who am i=%x should be %x\n\r$",0x1C,I2C_WriteReadBytes(I2C1,0x1C,FXOS8700CQ_WHOAMI,1),FXOS8700CQ_WHOAMI_VAL);
					printf("FXOS address=%x who am i=%x should be %x\n\r$",0x1D,I2C_WriteReadBytes(I2C1,0x1D,FXOS8700CQ_WHOAMI,1),FXOS8700CQ_WHOAMI_VAL);
					printf("FXOS address=%x who am i=%x should be %x\n\r$",0x1E,I2C_WriteReadBytes(I2C1,0x1E,FXOS8700CQ_WHOAMI,1),FXOS8700CQ_WHOAMI_VAL);
					printf("FXOS address=%x who am i=%x should be %x\n\r$",0x1F,I2C_WriteReadBytes(I2C1,0x1F,FXOS8700CQ_WHOAMI,1),FXOS8700CQ_WHOAMI_VAL);
					break;
				case '7':
					printf("\n\rToggle PA6 and PA7 - e to exit\n\r");
					while((inchar2=GetChar())!='e') {
						PA7HIGH;
						delay_us(5);
						PA7LOW;
						delay_us(10);
						PA6HIGH;
						delay_us(1);
						PA6LOW;
						delay_us(7);
						}
					PutChar('$');
					break;
				case CR:
					printf("\n\r$");
					break;
				}
			}
	  Toggle_Leds(100);
  	  }
}

#ifdef USE_FULL_ASSERT
/*******************************************************************************
* Function Name  : assert_failed
* Description    : Reports the name of the source file and the source line number
*                  where the assert_param error has occurred.
* Input          : - file: pointer to the source file name
*                  - line: assert_param error line source number
* Output         : None
* Return         : None
*******************************************************************************/
void assert_failed(uint8_t* file, uint32_t line)
{
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while (1)
  {
	  printf("Wrong parameters value: file %s on line %d\r\n", file, line);
  }
}
#endif


